#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Processor::Queue::Folder module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 46;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Processor::Queue::Folder';

        use_ok('Tester');
        use_ok('Mentat::Message::Factory');
        use_ok($TESTED_CLASS);

        $ENV{PATH} = '/bin:/usr/bin';
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
### $Mentat::Processor::Queue::Folder::VERSION

my $SPOOLDIR = '../../spool/';
mkdir($SPOOLDIR.'incoming');
mkdir($SPOOLDIR.'pending');
mkdir($SPOOLDIR.'done');
mkdir($SPOOLDIR.'tmp');
mkdir($SPOOLDIR.'errors');

### Attempting to create Mentat::Processor::Queue::Folder instance...

my $instance = $TESTED_CLASS->new(incoming_folder => $SPOOLDIR.'incoming',
                                  pending_folder  => $SPOOLDIR.'pending',
                                  done_folder     => $SPOOLDIR.'done',
                                  temp_folder     => $SPOOLDIR.'tmp',
                                  error_folder    => $SPOOLDIR.'errors');
isa_ok($instance, $TESTED_CLASS);

### Viewing the created instance: $instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D(2);

my $msg1 = Mentat::Message::Factory->from_file('./data/alert01.idea');
my $msg2 = Mentat::Message::Factory->from_file('./data/alert02.idea');
my $msg3 = Mentat::Message::Factory->from_file('./data/alert03.idea');
my $msg4 = Mentat::Message::Factory->from_file('./data/alert04.idea');
my $msg5 = Mentat::Message::Factory->from_file('./data/alert05.idea');
my $msg6 = Mentat::Message::Factory->from_file('./data/alert06.idea');

my $id1 = $instance->enqueue($msg1);
my $id2 = $instance->enqueue($msg2);
my $id3 = $instance->enqueue($msg3);
my $id4 = $instance->enqueue($msg4);
my $id5 = $instance->enqueue($msg5);
my $id6 = $instance->enqueue($msg6);

diag("Msg1: $id1->[0]") if Tester::D();
diag("Msg2: $id2->[0]") if Tester::D();
diag("Msg3: $id3->[0]") if Tester::D();
diag("Msg4: $id4->[0]") if Tester::D();
diag("Msg5: $id5->[0]") if Tester::D();
diag("Msg6: $id6->[0]") if Tester::D();

ok(-f ''.$SPOOLDIR.'incoming/'.$id1->[0], "Check test A01");
ok(-f ''.$SPOOLDIR.'incoming/'.$id2->[0], "Check test A02");
ok(-f ''.$SPOOLDIR.'incoming/'.$id3->[0], "Check test A03");
ok(-f ''.$SPOOLDIR.'incoming/'.$id4->[0], "Check test A04");
ok(-f ''.$SPOOLDIR.'incoming/'.$id5->[0], "Check test A05");
ok(-f ''.$SPOOLDIR.'incoming/'.$id6->[0], "Check test A06");

ok($instance->is_incoming($id1->[0]), "Check test X01");
ok($instance->is_incoming($id2->[0]), "Check test X02");
ok($instance->is_incoming($id3->[0]), "Check test X03");
ok($instance->is_incoming($id4->[0]), "Check test X04");
ok($instance->is_incoming($id5->[0]), "Check test X05");
ok($instance->is_incoming($id6->[0]), "Check test X06");

is($instance->count_incoming(), 6, "Check test B01");
is($instance->count_pending(),  0, "Check test B02");

my $r1 = $instance->dequeue();
is($instance->count_incoming(), 5, "Check test B03");
is($instance->count_pending(),  1, "Check test B04");

my $r2 = $instance->dequeue();
is($instance->count_incoming(), 4, "Check test B05");
is($instance->count_pending(),  2, "Check test B06");

my $r3 = $instance->dequeue();
is($instance->count_incoming(), 3, "Check test B07");
is($instance->count_pending(),  3, "Check test B08");

my $r4 = $instance->dequeue();
is($instance->count_incoming(), 2, "Check test B09");
is($instance->count_pending(),  4, "Check test B10");

my $r5 = $instance->dequeue();
is($instance->count_incoming(), 1, "Check test B11");
is($instance->count_pending(),  5, "Check test B12");

my $r6 = $instance->dequeue();
is($instance->count_incoming(), 0, "Check test B13");
is($instance->count_pending(),  6, "Check test B14");

ok(-f ''.$SPOOLDIR.'pending/'.$r1->[0], "Check test C01");
ok(-f ''.$SPOOLDIR.'pending/'.$r2->[0], "Check test C02");
ok(-f ''.$SPOOLDIR.'pending/'.$r3->[0], "Check test C03");
ok(-f ''.$SPOOLDIR.'pending/'.$r4->[0], "Check test C04");
ok(-f ''.$SPOOLDIR.'pending/'.$r5->[0], "Check test C05");
ok(-f ''.$SPOOLDIR.'pending/'.$r6->[0], "Check test C06");

$instance->banish($r1->[0]);
$instance->banish($r2->[0]);
ok(-f ''.$SPOOLDIR.'errors/'.$r1->[0], "Check test D01");
ok(-f ''.$SPOOLDIR.'errors/'.$r2->[0], "Check test D02");

$instance->commit($r3->[0]);
$instance->commit($r4->[0]);
ok(-f ''.$SPOOLDIR.'done/'.$r3->[0], "Check test E01");
ok(-f ''.$SPOOLDIR.'done/'.$r4->[0], "Check test E02");

$instance->remove($r5->[0]);
$instance->remove($r6->[0]);
is($instance->count_incoming(), 0, "Check test B15");
is($instance->count_pending(),  0, "Check test B16");

#my $msg7 = Mentat::Message::Factory->from_file('./spool/done/'.$id2->[0].'.DONE', 'idea');
#is($msg7->path_value('Alert/@messageid'), $msg3->path_value('Alert/@messageid'), 'Rewrite check test 01');

my $fn;
my $cnt = 0;
for $fn (
    ''.$SPOOLDIR.'errors/'.$r1->[0],
    ''.$SPOOLDIR.'errors/'.$r2->[0],
    ''.$SPOOLDIR.'done/'.$r3->[0],
    ''.$SPOOLDIR.'done/'.$r4->[0],
)
{
    if ($fn =~ /(.*)/) { $fn = $1 };
    $cnt++;
    is(unlink($fn), 1, 'Remove test 0'.$cnt);
}

diag("Dequeue check\n");
while(my $x = $instance->dequeue()) {
    diag("Dequeueing\n");
}
diag("Dequeue check done\n");

rmdir($SPOOLDIR.'incoming');
rmdir($SPOOLDIR.'pending');
rmdir($SPOOLDIR.'done');
rmdir($SPOOLDIR.'tmp');
rmdir($SPOOLDIR.'errors');
