#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Stats::Charter module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Time::HiRes qw(time);
use Test::More tests => 15;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Stats::Charter';

        use_ok('Tester');
        use_ok($TESTED_CLASS);

        $ENV{PATH} = '/bin:/usr/bin';
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
### $Mentat::Stats::Charter::VERSION

my $CHARTS_DIR = './spool';

### Attempting to create Mentat::Stats::Charter instance...
my $instance = $TESTED_CLASS->new(charts_dir => $CHARTS_DIR);
isa_ok($instance, $TESTED_CLASS);

### Viewing the created instance: $instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D(2);

my $charts = {
    'analyzers' => {
        'data' => [
            [
                'LaBrea',
                'Nemea',
                'HostStats',
                'HPScan',
                'DionaeaTUL',
                'KippoTUL',
                'Dionaea',
                'Kippo',
                'Fail2Ban'
            ],
            [
                260220,
                18614,
                13883,
                6808,
                629,
                315,
                75,
                59,
                13
            ]
        ],
    },
    'delay' => {
        'data' => [
            [
                1,
                2,
                3,
                4,
                5,
                6,
                7,
                8,
                9,
                10,
                11,
                12,
                13,
                14,
                15,
                16,
                17,
                18,
                19,
                20,
                21,
                22,
                23,
                24,
                25,
                26,
                27,
                28,
                29,
                30,
                31
            ],
            [
                489,
                495,
                495,
                497,
                565,
                565,
                1006,
                505,
                509,
                845,
                469,
                470,
                778,
                561,
                566,
                566,
                891,
                533,
                539,
                544,
                545,
                551,
                561,
                504,
                505,
                505,
                505,
                819,
                505,
                505,
                819
            ]
        ],
    },
};

my ($start, $stop, $duration);
$start = time;
$instance->chart('bar',     $charts->{delay}, 'test_delay', 'dur');
$instance->chart('sl_bar',  $charts->{delay}, 'test_delay', 'dur');
$instance->chart('sls_bar', $charts->{delay}, 'test_delay', 'dur');

$instance->chart('pie',     $charts->{analyzers}, 'test_analyzers', 'fn');
$instance->chart('sl_pie',  $charts->{analyzers}, 'test_analyzers', 'fn');
$instance->chart('sls_pie', $charts->{analyzers}, 'test_analyzers', 'fn');
$stop = time;
$duration = $stop - $start;
diag("GD::Graph charting duration: ".$duration."s") if Tester::D(2);
diag(Dumper($charts)) if Tester::D(2);

is(-f $CHARTS_DIR.'/'.$charts->{delay}->{file},     1, "Chart existence test 01");
is(-f $CHARTS_DIR.'/'.$charts->{delay}->{file_sl},  1, "Chart existence test 02");
is(-f $CHARTS_DIR.'/'.$charts->{delay}->{file_sls}, 1, "Chart existence test 03");
is(-f $CHARTS_DIR.'/'.$charts->{analyzers}->{file},     1, "Chart existence test 04");
is(-f $CHARTS_DIR.'/'.$charts->{analyzers}->{file_sl},  1, "Chart existence test 05");
is(-f $CHARTS_DIR.'/'.$charts->{analyzers}->{file_sls}, 1, "Chart existence test 06");

if (1) {
    is(unlink($CHARTS_DIR.'/'.$charts->{delay}->{file}),     1, "Chart removal test 01");
    is(unlink($CHARTS_DIR.'/'.$charts->{delay}->{file_sl}),  1, "Chart removal test 02");
    is(unlink($CHARTS_DIR.'/'.$charts->{delay}->{file_sls}), 1, "Chart removal test 03");
    is(unlink($CHARTS_DIR.'/'.$charts->{analyzers}->{file}),     1, "Chart removal test 04");
    is(unlink($CHARTS_DIR.'/'.$charts->{analyzers}->{file_sl}),  1, "Chart removal test 05");
    is(unlink($CHARTS_DIR.'/'.$charts->{analyzers}->{file_sls}), 1, "Chart removal test 06");
}
